#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <g3dviewer_plugin.h>

int image_expand(G3DImage *image, guint newwidth, guint newheight)
{
  guint y, size, off;
  guchar *newptr;

  g_return_val_if_fail(image != NULL, EXIT_FAILURE);
  g_return_val_if_fail(image->width <= newwidth, EXIT_FAILURE);
  g_return_val_if_fail(image->height <= newheight, EXIT_FAILURE);

  size = newwidth * newheight * (image->depth / 8);

  newptr = g_malloc0(size);

  if(newptr == NULL)
  {
    g_printerr("ERROR: image_expand: resizing failed (to %ux%ux%ubit)\n",
               newwidth, newheight, image->depth);
    return EXIT_FAILURE;
  }

  size = (newwidth - image->width) * (image->depth / 8);
  for(y=0; y<image->height; y++)
  {
    off = (y * newwidth) * (image->depth / 8);
    memcpy(newptr + off,
           image->pixeldata + (y * image->width * (image->depth/8)),
           size);
  }

  g_free(image->pixeldata);
  image->pixeldata = newptr;
  image->width = newwidth;
  image->height = newheight;

  return EXIT_SUCCESS;
}

